@@ -1,4 +1,20 @@
 /*
+ * Copyright (c) 2020-2021 illiliti <illiliti@protonmail.com>
+ * SPDX-License-Identifier: ISC
+ * 
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ *
  * this helper pretty similar to helper.sh, but it
  * doesn't write unrelated variables to file (e.g PWD or PATH)
  *
@@ -15,6 +31,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <limits.h>
+#include <sys/stat.h>
 
 int main(int argc, char **argv)
 {
@@ -32,7 +49,7 @@ int main(int argc, char **argv)
         break;
     default:
         fprintf(stderr, "usage: %s [dir]\n", argv[0]);
-        return 1;
+        return 2;
     }
 
     snprintf(path, sizeof(path), "%s/uevent.XXXXXX", dir);
@@ -44,8 +61,8 @@ int main(int argc, char **argv)
     }
 
     for (i = 0; environ[i]; i++) {
-        if (strncmp(environ[i], "PATH", 4) == 0 ||
-            strncmp(environ[i], "HOME", 4) == 0) {
+        if (strncmp(environ[i], "PATH=", 5) == 0 ||
+            strncmp(environ[i], "HOME=", 5) == 0) {
             continue;
         }
 
@@ -58,6 +75,7 @@ int main(int argc, char **argv)
         }
     }
 
+    fchmod(fd, 0444);
     close(fd);
     return 0;
 }
