@@ -10,4 +10,4 @@
 
 # NOTE: writing variables to file (e.g PWD or PATH)
 # that are not related to uevent properties is harmless
-env > "${1:-/tmp/.libudev-zero}/uevent.$$"
+exec env > "${1:-/tmp/.libudev-zero}/uevent.$$"
