@@ -1,3 +1,20 @@
+/*
+ * Copyright (c) 2020-2021 illiliti <illiliti@protonmail.com>
+ * SPDX-License-Identifier: ISC
+ * 
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -9,7 +26,17 @@
 #include "udev.h"
 #include "udev_list.h"
 
-#define BITS_MAX 96
+// sizeof(unsigned long) == 4 * 8 == 32 on 32-bit systems.
+// sizeof(unsigned long) == 8 * 8 == 64 on 64-bit systems.
+#ifndef LONG_BIT
+#define LONG_BIT (sizeof(unsigned long) * 8)
+#endif
+
+// https://kernel.org/doc/html/latest/input/ff.html#querying-device-capabilities
+// https://kernel.org/doc/html/latest/input/input-programming.html#bits-to-longs-bit-word-bit-mask
+#define BIT_WORD(x) ((x) / LONG_BIT)
+#define BIT_MASK(x) (1UL << ((x) % LONG_BIT))
+#define BITS_TO_LONGS(x) (((x) + LONG_BIT - 1) / LONG_BIT)
 
 #ifndef INPUT_PROP_POINTING_STICK
 #define INPUT_PROP_POINTING_STICK 0x05
@@ -77,7 +104,7 @@ dev_t udev_device_get_devnum(struct udev_device *udev_device)
     major = udev_device_get_property_value(udev_device, "MAJOR");
     minor = udev_device_get_property_value(udev_device, "MINOR");
 
-    if (!major && !minor) {
+    if (!major || !minor) {
         return makedev(0, 0);
     }
 
@@ -212,11 +239,10 @@ const char *udev_device_get_property_value(struct udev_device *udev_device, cons
 const char *udev_device_get_sysattr_value(struct udev_device *udev_device, const char *sysattr)
 {
     struct udev_list_entry *list_entry;
-    char data[1024], path[PATH_MAX];
+    char data[4096], path[PATH_MAX];
     struct stat st;
     size_t len;
     FILE *file;
-    char *pos;
 
     if (!udev_device || !sysattr) {
         return NULL;
@@ -240,19 +266,21 @@ const char *udev_device_get_sysattr_value(struct udev_device *udev_device, const
         return NULL;
     }
 
-    len = fread(data, 1, sizeof(data), file);
+    // TODO dynamic allocation of data
+    len = fread(data, 1, sizeof(data) - 1, file);
 
-    if (len != sizeof(data) && ferror(file)) {
+    if (len != sizeof(data) - 1 && ferror(file)) {
         fclose(file);
         return NULL;
     }
 
-    if ((pos = memchr(data, '\n', len))) {
-        len = pos - data;
-    }
-
     fclose(file);
     data[len] = '\0';
+
+    while (len-- > 0 && data[len] == '\n') {
+        data[len] = '\0';
+    }
+
     list_entry = udev_list_entry_add(&udev_device->sysattrs, sysattr, data, 0);
     return udev_list_entry_get_value(list_entry);
 }
@@ -292,12 +320,12 @@ int udev_device_set_sysattr_value(struct udev_device *udev_device, const char *s
     return 0;
 }
 
-static char *udev_device_read_symlink(struct udev_device *udev_device, const char *name)
+static char *read_symlink(const char *syspath, const char *name)
 {
     char link[PATH_MAX], path[PATH_MAX];
     ssize_t len;
 
-    snprintf(path, sizeof(path), "%s/%s", udev_device_get_syspath(udev_device), name);
+    snprintf(path, sizeof(path), "%s/%s", syspath, name);
     len = readlink(path, link, sizeof(link));
 
     if (len == -1) {
@@ -308,7 +336,7 @@ static char *udev_device_read_symlink(struct udev_device *udev_device, const cha
     return strdup(strrchr(link, '/') + 1);
 }
 
-static void udev_device_set_properties_from_uevent(struct udev_device *udev_device)
+static void set_properties_from_uevent(struct udev_device *udev_device)
 {
     char line[LINE_MAX], path[PATH_MAX], devnode[PATH_MAX];
     FILE *file;
@@ -324,7 +352,7 @@ static void udev_device_set_properties_from_uevent(struct udev_device *udev_devi
     while (fgets(line, sizeof(line), file)) {
         line[strlen(line) - 1] = '\0';
 
-        if (strncmp(line, "DEVNAME", 7) == 0) {
+        if (strncmp(line, "DEVNAME=", 8) == 0) {
             snprintf(devnode, sizeof(devnode), "/dev/%s", line + 8);
             udev_list_entry_add(&udev_device->properties, "DEVNAME", devnode, 0);
         }
@@ -337,53 +365,44 @@ static void udev_device_set_properties_from_uevent(struct udev_device *udev_devi
     fclose(file);
 }
 
-static int populate_bit(unsigned long *arr, const char *str)
+static void make_bit(unsigned long *arr, int cnt, const char *str)
 {
-    char *beg, *end;
+    size_t len;
     int i;
 
     if (!str) {
-        return 0;
+        return;
     }
 
-    // TODO drop strdup ?
-    beg = end = strdup(str);
+    for (i = 0, len = strlen(str); len > 0; len--) {
+        if (str[len] == ' ') {
+            arr[i++] = strtoul(str + len + 1, NULL, 16);
 
-    if (!beg) {
-        return 0;
-    }
-
-    for (i = 0; end[0] != '\0' && i < BITS_MAX; i++) {
-        arr[i] = strtoul(end, &end, 16);
-    }
-
-    free(beg);
-    return i;
-}
-
-static int find_bit(unsigned long *arr, int cnt, unsigned long bit)
-{
-    int i;
-
-    for (i = 0; i < cnt; i++) {
-        if (arr[i] & (1UL << (bit % LONG_BIT))) {
-            return 1;
+            if (i == cnt) {
+                return;
+            }
         }
     }
 
-    return 0;
+    arr[i] = strtoul(str, NULL, 16);
 }
 
-static void udev_device_set_properties_from_evdev(struct udev_device *udev_device)
+static int test_bit(unsigned long *arr, unsigned long bit)
 {
-    int ev_cnt, rel_cnt, key_cnt, abs_cnt, prop_cnt;
-    unsigned long prop_bits[BITS_MAX] = {0};
-    unsigned long abs_bits[BITS_MAX] = {0};
-    unsigned long rel_bits[BITS_MAX] = {0};
-    unsigned long key_bits[BITS_MAX] = {0};
-    unsigned long ev_bits[BITS_MAX] = {0};
+    return !!(arr[BIT_WORD(bit)] & BIT_MASK(bit));
+}
+
+static void set_properties_from_evdev(struct udev_device *udev_device)
+{
+    // https://kernel.org/doc/html/latest/driver-api/input.html#c.input_dev
+    unsigned long prop_bits[BITS_TO_LONGS(INPUT_PROP_CNT)] = {0};
+    unsigned long abs_bits[BITS_TO_LONGS(ABS_CNT)] = {0};
+    unsigned long rel_bits[BITS_TO_LONGS(REL_CNT)] = {0};
+    unsigned long key_bits[BITS_TO_LONGS(KEY_CNT)] = {0};
+    unsigned long ev_bits[BITS_TO_LONGS(EV_CNT)] = {0};
     struct udev_device *parent;
     const char *subsystem;
+    unsigned long bit;
 
     subsystem = udev_device_get_subsystem(udev_device);
 
@@ -405,67 +424,86 @@ static void udev_device_set_properties_from_evdev(struct udev_device *udev_devic
         parent = udev_device_get_parent_with_subsystem_devtype(parent, "input", NULL);
     }
 
-    ev_cnt = populate_bit(ev_bits, udev_device_get_property_value(parent, "EV"));
-    abs_cnt = populate_bit(abs_bits, udev_device_get_property_value(parent, "ABS"));
-    rel_cnt = populate_bit(rel_bits, udev_device_get_property_value(parent, "REL"));
-    key_cnt = populate_bit(key_bits, udev_device_get_property_value(parent, "KEY"));
-    prop_cnt = populate_bit(prop_bits, udev_device_get_property_value(parent, "PROP"));
+    make_bit(ev_bits, sizeof(ev_bits) / sizeof(ev_bits[0]), udev_device_get_property_value(parent, "EV"));
+    make_bit(abs_bits, sizeof(abs_bits) / sizeof(abs_bits[0]), udev_device_get_property_value(parent, "ABS"));
+    make_bit(rel_bits, sizeof(rel_bits) / sizeof(rel_bits[0]), udev_device_get_property_value(parent, "REL"));
+    make_bit(key_bits, sizeof(key_bits) / sizeof(key_bits[0]), udev_device_get_property_value(parent, "KEY"));
+    make_bit(prop_bits, sizeof(prop_bits) / sizeof(prop_bits[0]), udev_device_get_property_value(parent, "PROP"));
 
     udev_list_entry_add(&udev_device->properties, "ID_INPUT", "1", 0);
 
-    if (find_bit(prop_bits, prop_cnt, INPUT_PROP_POINTING_STICK)) {
+    if (test_bit(prop_bits, INPUT_PROP_POINTING_STICK)) {
         udev_list_entry_add(&udev_device->properties, "ID_INPUT_POINTINGSTICK", "1", 0);
     }
 
-    if (find_bit(prop_bits, prop_cnt, INPUT_PROP_ACCELEROMETER)) {
+    if (test_bit(prop_bits, INPUT_PROP_ACCELEROMETER)) {
         udev_list_entry_add(&udev_device->properties, "ID_INPUT_ACCELEROMETER", "1", 0);
     }
 
-    if (find_bit(ev_bits, ev_cnt, EV_SW)) {
+    if (test_bit(ev_bits, EV_SW)) {
         udev_list_entry_add(&udev_device->properties, "ID_INPUT_SWITCH", "1", 0);
     }
 
-    if (find_bit(ev_bits, ev_cnt, EV_REL)) {
-        if (find_bit(rel_bits, rel_cnt, REL_Y) && find_bit(rel_bits, rel_cnt, REL_X) &&
-            find_bit(key_bits, key_cnt, BTN_MOUSE)) {
+    if (test_bit(ev_bits, EV_REL)) {
+        if (test_bit(rel_bits, REL_Y) && test_bit(rel_bits, REL_X) &&
+            test_bit(key_bits, BTN_MOUSE)) {
             udev_list_entry_add(&udev_device->properties, "ID_INPUT_MOUSE", "1", 0);
         }
     }
-    else if (find_bit(ev_bits, ev_cnt, EV_ABS)) {
-        if (find_bit(key_bits, key_cnt, BTN_SELECT) || find_bit(key_bits, key_cnt, BTN_TR) ||
-            find_bit(key_bits, key_cnt, BTN_START) || find_bit(key_bits, key_cnt, BTN_TL)) {
-            udev_list_entry_add(&udev_device->properties, "ID_INPUT_JOYSTICK", "1", 0);
+    else if (test_bit(ev_bits, EV_ABS)) {
+        if (test_bit(key_bits, BTN_SELECT) || test_bit(key_bits, BTN_TR) ||
+            test_bit(key_bits, BTN_START) || test_bit(key_bits, BTN_TL)) {
+            if (test_bit(key_bits, BTN_TOUCH)) {
+                udev_list_entry_add(&udev_device->properties, "ID_INPUT_TOUCHSCREEN", "1", 0);
+            }
+            else {
+                udev_list_entry_add(&udev_device->properties, "ID_INPUT_JOYSTICK", "1", 0);
+            }
         }
-        else if (find_bit(abs_bits, abs_cnt, ABS_Y) && find_bit(abs_bits, abs_cnt, ABS_X)) {
-            if (find_bit(abs_bits, abs_cnt, ABS_Z) && !find_bit(ev_bits, ev_cnt, EV_KEY)) {
+        else if (test_bit(abs_bits, ABS_Y) && test_bit(abs_bits, ABS_X)) {
+            if (test_bit(abs_bits, ABS_Z) && !test_bit(ev_bits, EV_KEY)) {
                 udev_list_entry_add(&udev_device->properties, "ID_INPUT_ACCELEROMETER", "1", 0);
             }
-            else if (find_bit(key_bits, key_cnt, BTN_STYLUS) || find_bit(key_bits, key_cnt, BTN_TOOL_PEN)) {
+            else if (test_bit(key_bits, BTN_STYLUS) || test_bit(key_bits, BTN_TOOL_PEN)) {
                 udev_list_entry_add(&udev_device->properties, "ID_INPUT_TABLET", "1", 0);
             }
-            else if (find_bit(key_bits, key_cnt, BTN_TOUCH)) {
-                if (find_bit(key_bits, key_cnt, BTN_TOOL_FINGER)) {
+            else if (test_bit(key_bits, BTN_TOUCH)) {
+                if (test_bit(key_bits, BTN_TOOL_FINGER)) {
                     udev_list_entry_add(&udev_device->properties, "ID_INPUT_TOUCHPAD", "1", 0);
                 }
                 else {
                     udev_list_entry_add(&udev_device->properties, "ID_INPUT_TOUCHSCREEN", "1", 0);
                 }
             }
-            else if (find_bit(key_bits, key_cnt, BTN_MOUSE)) {
+            else if (test_bit(key_bits, BTN_MOUSE)) {
                 udev_list_entry_add(&udev_device->properties, "ID_INPUT_MOUSE", "1", 0);
             }
         }
     }
-    else if (find_bit(ev_bits, ev_cnt, EV_KEY)) {
+
+    if (!test_bit(ev_bits, EV_KEY)) {
+        return;
+    }
+
+    // iterate over key bits until BTN_* block is hit and test if bitmask has KEY_*.
+    // this way, we can check if device has keys or buttons.
+    // https://github.com/torvalds/linux/blob/f5b6eb1e018203913dfefcf6fa988649ad11ad6e/include/uapi/linux/input-event-codes.h#L76-L338
+    for (bit = KEY_ESC; bit < BTN_MISC; bit++) {
+        if (!test_bit(key_bits, bit)) {
+            continue;
+        }
+
         udev_list_entry_add(&udev_device->properties, "ID_INPUT_KEY", "1", 0);
 
-        if (find_bit(key_bits, key_cnt, KEY_ENTER)) {
+        if (test_bit(key_bits, KEY_ENTER)) {
             udev_list_entry_add(&udev_device->properties, "ID_INPUT_KEYBOARD", "1", 0);
         }
+
+        return;
     }
 }
 
-static void udev_device_set_properties_from_props(struct udev_device *udev_device)
+static void set_properties_from_props(struct udev_device *udev_device)
 {
     const char *sysname, *subsystem;
     struct udev_device *parent;
@@ -526,8 +564,8 @@ struct udev_device *udev_device_new_from_syspath(struct udev *udev, const char *
     udev_list_entry_add(&udev_device->properties, "DEVPATH", path + 4, 0);
 
     sysname = strrchr(path, '/') + 1;
-    driver = udev_device_read_symlink(udev_device, "driver");
-    subsystem = udev_device_read_symlink(udev_device, "subsystem");
+    driver = read_symlink(path, "driver");
+    subsystem = read_symlink(path, "subsystem");
 
     udev_list_entry_add(&udev_device->properties, "SUBSYSTEM", subsystem, 0);
     udev_list_entry_add(&udev_device->properties, "SYSNAME", sysname, 0);
@@ -540,9 +578,9 @@ struct udev_device *udev_device_new_from_syspath(struct udev *udev, const char *
         }
     }
 
-    udev_device_set_properties_from_uevent(udev_device);
-    udev_device_set_properties_from_evdev(udev_device);
-    udev_device_set_properties_from_props(udev_device);
+    set_properties_from_uevent(udev_device);
+    set_properties_from_evdev(udev_device);
+    set_properties_from_props(udev_device);
 
     free(driver);
     free(subsystem);
@@ -553,7 +591,7 @@ struct udev_device *udev_device_new_from_devnum(struct udev *udev, char type, de
 {
     char path[PATH_MAX];
 
-    if (!udev || !type || !devnum) {
+    if (!udev) {
         return NULL;
     }
 
@@ -596,11 +634,11 @@ struct udev_device *udev_device_new_from_file(struct udev *udev, const char *pat
 {
     char line[LINE_MAX], syspath[PATH_MAX], devnode[PATH_MAX];
     struct udev_device *udev_device;
-    char *sysname = NULL;
     struct stat st;
+    char *sysname;
     FILE *file;
+    int i, cnt;
     char *pos;
-    int i;
 
     if (stat(path, &st) != 0 || st.st_size > 8192) {
         return NULL;
@@ -619,6 +657,8 @@ struct udev_device *udev_device_new_from_file(struct udev *udev, const char *pat
         return NULL;
     }
 
+    cnt = 0;
+
     udev_device->udev = udev;
     udev_device->refcount = 1;
     udev_device->parent = NULL;
@@ -629,7 +669,7 @@ struct udev_device *udev_device_new_from_file(struct udev *udev, const char *pat
     while (fgets(line, sizeof(line), file)) {
         line[strlen(line) - 1] = '\0';
 
-        if (strncmp(line, "DEVPATH", 7) == 0) {
+        if (strncmp(line, "DEVPATH=", 8) == 0) {
             snprintf(syspath, sizeof(syspath), "/sys%s", line + 8);
             udev_list_entry_add(&udev_device->properties, "SYSPATH", syspath, 0);
             udev_list_entry_add(&udev_device->properties, "DEVPATH", line + 8, 0);
@@ -643,26 +683,45 @@ struct udev_device *udev_device_new_from_file(struct udev *udev, const char *pat
                     break;
                 }
             }
+
+            cnt++;
         }
-        else if (strncmp(line, "DEVNAME", 7) == 0) {
+        else if (strncmp(line, "DEVNAME=", 8) == 0) {
             snprintf(devnode, sizeof(devnode), "/dev/%s", line + 8);
             udev_list_entry_add(&udev_device->properties, "DEVNAME", devnode, 0);
         }
-        else if ((pos = strchr(line, '='))) {
+        else {
+            pos = strchr(line, '=');
+
+            // file is malformed, abort here.
+            if (!pos) {
+                cnt = 0;
+                break;
+            }
+
             *pos = '\0';
+
+            if (strncmp(line, "SUBSYSTEM", 9) == 0 ||
+                strncmp(line, "ACTION", 6) == 0 ||
+                strncmp(line, "SEQNUM", 6) == 0) {
+                cnt++;
+            }
+
             udev_list_entry_add(&udev_device->properties, line, pos + 1, 0);
         }
     }
 
     fclose(file);
 
-    if (!sysname) {
+    // https://freedesktop.org/software/systemd/man/udev_device_new_from_environment.html
+    // > The keys DEVPATH, SUBSYSTEM, ACTION, and SEQNUM are mandatory.
+    if (cnt != 4) {
         udev_device_unref(udev_device);
         return NULL;
     }
 
-    udev_device_set_properties_from_props(udev_device);
-    udev_device_set_properties_from_evdev(udev_device);
+    set_properties_from_props(udev_device);
+    set_properties_from_evdev(udev_device);
     return udev_device;
 }
 
