@@ -1,3 +1,20 @@
+/*
+ * Copyright (c) 2020-2021 illiliti <illiliti@protonmail.com>
+ * SPDX-License-Identifier: ISC
+ * 
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <dirent.h>
@@ -73,7 +90,7 @@ int udev_enumerate_add_match_sysname(struct udev_enumerate *udev_enumerate, cons
     return 0;
 }
 
-static int udev_enumerate_filter_subsystem(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
+static int filter_subsystem(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
 {
     struct udev_list_entry *list_entry;
     const char *subsystem;
@@ -110,7 +127,7 @@ static int udev_enumerate_filter_subsystem(struct udev_enumerate *udev_enumerate
     return 1;
 }
 
-static int udev_enumerate_filter_sysname(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
+static int filter_sysname(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
 {
     struct udev_list_entry *list_entry;
     const char *sysname;
@@ -133,7 +150,7 @@ static int udev_enumerate_filter_sysname(struct udev_enumerate *udev_enumerate,
     return 0;
 }
 
-static int udev_enumerate_filter_property(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
+static int filter_property(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
 {
     const char *property, *property2, *value, *value2;
     struct udev_list_entry *list_entry, *list_entry2;
@@ -176,7 +193,7 @@ static int udev_enumerate_filter_property(struct udev_enumerate *udev_enumerate,
     return 0;
 }
 
-static int udev_enumerate_filter_sysattr(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
+static int filter_sysattr(struct udev_enumerate *udev_enumerate, struct udev_device *udev_device)
 {
     struct udev_list_entry *list_entry;
     const char *sysattr, *value;
@@ -222,7 +239,7 @@ static int udev_enumerate_filter_sysattr(struct udev_enumerate *udev_enumerate,
     return 1;
 }
 
-static void *udev_enumerate_add_device(void *ptr)
+static void *add_device(void *ptr)
 {
     struct udev_enumerate_thread *thread = ptr;
     struct udev_device *udev_device;
@@ -233,10 +250,10 @@ static void *udev_enumerate_add_device(void *ptr)
         return NULL;
     }
 
-    if (!udev_enumerate_filter_subsystem(thread->udev_enumerate, udev_device) ||
-        !udev_enumerate_filter_sysname(thread->udev_enumerate, udev_device) ||
-        !udev_enumerate_filter_property(thread->udev_enumerate, udev_device) ||
-        !udev_enumerate_filter_sysattr(thread->udev_enumerate, udev_device)) {
+    if (!filter_subsystem(thread->udev_enumerate, udev_device) ||
+        !filter_sysname(thread->udev_enumerate, udev_device) ||
+        !filter_property(thread->udev_enumerate, udev_device) ||
+        !filter_sysattr(thread->udev_enumerate, udev_device)) {
         udev_device_unref(udev_device);
         return NULL;
     }
@@ -254,28 +271,25 @@ static int filter_dot(const struct dirent *de)
     return de->d_name[0] != '.';
 }
 
-static int udev_enumerate_add_devices(struct udev_enumerate *udev_enumerate, const char *path)
+static int scan_devices(struct udev_enumerate *udev_enumerate, const char *path)
 {
     struct udev_enumerate_thread *thread;
     pthread_mutex_t mutex;
     struct dirent **de;
-    int cnt, i;
+    int ret, cnt, i;
 
     cnt = scandir(path, &de, filter_dot, NULL);
 
     if (cnt == -1) {
-        return -1;
+        return 0;
     }
 
+    ret = 1;
     thread = calloc(cnt, sizeof(struct udev_enumerate_thread));
 
     if (!thread) {
-        for (i = 0; i < cnt; i++) {
-            free(de[i]);
-        }
-
-        free(de);
-        return -1;
+        ret = 0;
+        goto free_de;
     }
 
     pthread_mutex_init(&mutex, NULL);
@@ -285,21 +299,27 @@ static int udev_enumerate_add_devices(struct udev_enumerate *udev_enumerate, con
         thread[i].udev_enumerate = udev_enumerate;
 
         snprintf(thread[i].path, sizeof(thread[i].path), "%s/%s", path, de[i]->d_name);
-        pthread_create(&thread[i].thread, NULL, udev_enumerate_add_device, &thread[i]);
+
+        if (pthread_create(&thread[i].thread, NULL, add_device, &thread[i]) != 0) {
+            ret = 0;
+            break;
+        }
     }
 
     for (i = 0; i < cnt; i++) {
         pthread_join(thread[i].thread, NULL);
     }
 
+    free(thread);
+    pthread_mutex_destroy(&mutex);
+
+free_de:
     for (i = 0; i < cnt; i++) {
         free(de[i]);
     }
 
     free(de);
-    free(thread);
-    pthread_mutex_destroy(&mutex);
-    return 0;
+    return ret;
 }
 
 int udev_enumerate_scan_devices(struct udev_enumerate *udev_enumerate)
@@ -312,7 +332,7 @@ int udev_enumerate_scan_devices(struct udev_enumerate *udev_enumerate)
     }
 
     for (i = 0; path[i]; i++) {
-        if (udev_enumerate_add_devices(udev_enumerate, path[i]) == -1) {
+        if (!scan_devices(udev_enumerate, path[i])) {
             return -1;
         }
     }
